/**********
 * 
 * diskmodel.h - 
 *     Constants and data structures used in modelling the
 *     behavior of an HP 97560 hard drive
 * 
 * Nils Nieuwejaar 1995-96
 *
 *********/

#ifndef DISKMODEL_H
#define DISKMODEL_H

#ifndef _HAVE_ULONG_T_
typedef unsigned long ulong_t;
#endif

struct disk_region {
  ulong_t lstart, lend;
  ulong_t pstart, pend;
  int length;
};

struct disk_address {
    int cylinder;
    int track;
    int head;
    ulong_t logical, physical;
};

struct disk_state {
    struct disk_address add;
    struct timeval last_time;
    struct timeval ready_time;
};

struct cache_state {
    int valid;
    int last;
    struct disk_address first;
    struct disk_address prefetch;
    struct timeval start;
    struct timeval done;
};

/* All parameters are from ruemmler:model2 and hp:97560 */

#define SECTOR_SIZE     512   /* bytes */
#define SEC_PER_TRACK   72
#define TRACKS_PER_CYL  19 
#define NUM_REGIONS     3

/* cylinder skew of the disk in sectors */
#define CYLINDERSKEW   18
/* track skew of the disk in sectors */
#define TRACKSKEW       8

/* Combine bus and controller overhead into a single number */
#define OVERHEAD        2300  /* microseconds */

#define HEAD_SWITCH     2500  /* microseconds */
#define CYLINDER_SWITCH 2500  /* microseconds */

/* 10 MB/sec -> bytes per us */
#define BUSSPEED          (10.0 * 1024.0 * 1024.0 / 1000000.0) 

#define DISKRPM            4002
#define USEC_PER_REV       (60.0 * 1000000.0 / DISKRPM)
#define USEC_PER_SECTOR    (USEC_PER_REV / SEC_PER_TRACK)

/* write fence size */
#define WRITEFENCE         (64 * 1024.0) /* bytes */
#define write_fence_time   ((ulong_t)(WRITEFENCE / BUSSPEED))  /* usec */

/* size of disk's on-board cache in KB */
#define CACHE_SIZE         128

/* size of disk's on-board cache in sectors */
#define CACHE_SLOTS        (CACHE_SIZE * 1024 / SECTOR_SIZE)

#endif

